@@ -23,7 +23,7 @@
 http://nbsphinx.rtfd.org/
 
 """
-__version__ = '0.2.11'
+__version__ = '0.2.13'
 
 import copy
 import json
@@ -49,6 +49,8 @@ _ipynbversion = 4
 # See nbconvert/exporters/html.py:
 DISPLAY_DATA_PRIORITY_HTML = (
     'application/javascript',
+    'application/vnd.jupyter.widget-view+json',
+    'application/vnd.jupyter.widget-state+json',
     'text/html',
     'text/markdown',
     'image/svg+xml',
@@ -156,6 +158,11 @@ RST_TEMPLATE = """
         var element = document.getElementById('{{ div_id }}');
 {{ output.data['application/javascript'] | indent | indent }}
         </script>
+{%- elif datatype.startswith('application') and datatype.endswith('+json') %}
+
+    .. raw:: html
+
+        <script type="{{ datatype }}">{{ output.data[datatype] | json_dumps }}</script>
 {%- elif datatype == 'ansi' %}
 
     .. rst-class:: highlight
@@ -233,6 +240,20 @@ RST_TEMPLATE = """
 {{ cell.source }}
 {% endif %}
 {% endblock rawcell %}
+
+
+{% block footer %}
+
+{% if 'application/vnd.jupyter.widget-state+json' in nb.metadata.widgets %}
+
+.. raw:: html
+
+    <script type="application/vnd.jupyter.widget-state+json">
+    {{ nb.metadata.widgets['application/vnd.jupyter.widget-state+json'] | json_dumps }}
+    </script>
+{% endif %}
+{{ super() }}
+{% endblock footer %}
 """
 
 
@@ -265,13 +286,15 @@ CSS_STRING = """
 
 /* remove conflicting styling from Sphinx themes */
 div.nbinput,
-div.nbinput > div,
-div.nbinput div[class^=highlight],
-div.nbinput div[class^=highlight] pre,
+div.nbinput div.prompt,
+div.nbinput div.input_area,
+div.nbinput div[class*=highlight],
+div.nbinput div[class*=highlight] pre,
 div.nboutput,
-div.nboutput > div,
-div.nboutput div[class^=highlight],
-div.nboutput div[class^=highlight] pre {
+div.nbinput div.prompt,
+div.nbinput div.output_area,
+div.nboutput div[class*=highlight],
+div.nboutput div[class*=highlight] pre {
     background: none;
     border: none;
     padding: 0 0;
@@ -280,7 +303,7 @@ div.nboutput div[class^=highlight] pre {
 }
 
 /* avoid gaps between output lines */
-div.nboutput div[class^=highlight] pre {
+div.nboutput div[class*=highlight] pre {
     line-height: normal;
 }
 
@@ -291,6 +314,13 @@ div.nboutput {
     display: flex;
     align-items: flex-start;
     margin: 0;
+    width: 100%%;
+}
+@media (max-width: %(nbsphinx_responsive_width)s) {
+    div.nbinput,
+    div.nboutput {
+        flex-direction: column;
+    }
 }
 
 /* input container */
@@ -304,38 +334,58 @@ div.nblast {
 }
 
 /* input prompt */
-div.nbinput > :first-child pre {
+div.nbinput div.prompt pre {
     color: #303F9F;
 }
 
 /* output prompt */
-div.nboutput > :first-child pre {
+div.nboutput div.prompt pre {
     color: #D84315;
 }
 
 /* all prompts */
-div.nbinput > :first-child[class^=highlight],
-div.nboutput > :first-child[class^=highlight],
-div.nboutput > :first-child {
+div.nbinput div.prompt,
+div.nboutput div.prompt {
     min-width: %(nbsphinx_prompt_width)s;
     padding-top: 0.4em;
     padding-right: 0.4em;
     text-align: right;
     flex: 0;
 }
+@media (max-width: %(nbsphinx_responsive_width)s) {
+    div.nbinput div.prompt,
+    div.nboutput div.prompt {
+        text-align: left;
+        padding: 0.4em;
+    }
+    div.nboutput div.prompt.empty {
+        padding: 0;
+    }
+}
+
+/* disable scrollbars on prompts */
+div.nbinput div.prompt pre,
+div.nboutput div.prompt pre {
+    overflow: hidden;
+}
 
 /* input/output area */
-div.nbinput > :nth-child(2)[class^=highlight],
-div.nboutput > :nth-child(2),
-div.nboutput > :nth-child(2)[class^=highlight] {
+div.nbinput div.input_area,
+div.nboutput div.output_area {
     padding: 0.4em;
     -webkit-flex: 1;
     flex: 1;
     overflow: auto;
 }
+@media (max-width: %(nbsphinx_responsive_width)s) {
+    div.nbinput div.input_area,
+    div.nboutput div.output_area {
+        width: 100%%;
+    }
+}
 
 /* input area */
-div.nbinput > :nth-child(2)[class^=highlight] {
+div.nbinput div.input_area {
     border: 1px solid #cfcfcf;
     border-radius: 2px;
     background: #f7f7f7;
@@ -352,7 +402,7 @@ div.nboutput div.math p {
 }
 
 /* standard error */
-div.nboutput  > :nth-child(2).stderr {
+div.nboutput div.output_area.stderr {
     background: #fdd;
 }
 
@@ -449,21 +499,17 @@ class Exporter(nbconvert.RSTExporter):
         self._codecell_lexer = codecell_lexer
         loader = jinja2.DictLoader({'nbsphinx-rst.tpl': RST_TEMPLATE})
         super(Exporter, self).__init__(
-            template_file='nbsphinx-rst', extra_loaders=[loader],
+            template_file='nbsphinx-rst.tpl', extra_loaders=[loader],
+            config= traitlets.config.Config(
+                {'HighlightMagicsPreprocessor': {'enabled': True}}),
             filters={
                 'markdown2rst': markdown2rst,
                 'get_empty_lines': _get_empty_lines,
                 'extract_toctree': _extract_toctree,
                 'get_output_type': _get_output_type,
+                'json_dumps': json.dumps,
             })
 
-    @property
-    def default_config(self):
-        c = traitlets.config.Config(
-            {'HighlightMagicsPreprocessor': {'enabled': True}})
-        c.merge(super(Exporter, self).default_config)
-        return c
-
     def from_notebook_node(self, nb, resources=None, **kw):
         nb = copy.deepcopy(nb)
         if resources is None:
@@ -479,13 +525,14 @@ class Exporter(nbconvert.RSTExporter):
         if execute not in ('always', 'never', 'auto'):
             raise ValueError('invalid execute option: {!r}'.format(execute))
         auto_execute = (
+            execute == 'auto' and
             # At least one code cell actually containing source code:
             any(c.source for c in nb.cells if c.cell_type == 'code') and
             # No outputs, not even a prompt number:
             not any(c.get('outputs') or c.get('execution_count')
                     for c in nb.cells if c.cell_type == 'code')
         )
-        if execute == 'always' or (execute == 'auto' and auto_execute):
+        if auto_execute or execute == 'always':
             allow_errors = nbsphinx_metadata.get(
                 'allow_errors', self._allow_errors)
             timeout = nbsphinx_metadata.get('timeout', self._timeout)
@@ -516,6 +563,8 @@ class NotebookParser(rst.Parser):
 
     """
 
+    supported = ()
+
     def get_transforms(self):
         """List of transforms for documents parsed by this parser."""
         return rst.Parser.get_transforms(self) + [
@@ -578,14 +627,15 @@ class CodeNode(docutils.nodes.Element):
     """A custom node that contains a literal_block node."""
 
     @classmethod
-    def create(cls, text, language='none'):
+    def create(cls, text, language='none', **kwargs):
         """Create a new CodeNode containing a literal_block node.
 
         Apparently, this cannot be done in CodeNode.__init__(), see:
         https://groups.google.com/forum/#!topic/sphinx-dev/0chv7BsYuW0
 
         """
-        node = docutils.nodes.literal_block(text, text, language=language)
+        node = docutils.nodes.literal_block(text, text, language=language,
+                                            **kwargs)
         return cls(text, node)
 
 
@@ -619,13 +669,14 @@ class NbInput(rst.Directive):
 
         # Input prompt
         text = 'In [{}]:'.format(execution_count if execution_count else ' ')
-        container += CodeNode.create(text)
+        container += CodeNode.create(text, classes=['prompt'])
         latex_prompt = text + ' '
 
         # Input code area
         text = '\n'.join(self.content.data)
         node = CodeNode.create(
-            text, language=self.arguments[0] if self.arguments else 'none')
+            text, language=self.arguments[0] if self.arguments else 'none',
+            classes=['input_area'])
         _set_empty_lines(node, self.options)
         node.attributes['latex_prompt'] = latex_prompt
         container += node
@@ -660,22 +711,23 @@ class NbOutput(rst.Directive):
         # Optional output prompt
         if execution_count:
             text = 'Out[{}]:'.format(execution_count)
-            container += CodeNode.create(text)
+            container += CodeNode.create(text, classes=['prompt'])
             latex_prompt = text + ' '
         else:
-            container += rst.nodes.container()  # empty container for HTML
+            # Empty container for HTML:
+            container += rst.nodes.container(classes=['prompt', 'empty'])
             latex_prompt = ''
 
         # Output area
         if outputtype == 'rst':
-            classes = [self.options.get('class', '')]
+            classes = [self.options.get('class', ''), 'output_area']
             output_area = docutils.nodes.container(classes=classes)
             self.state.nested_parse(self.content, self.content_offset,
                                     output_area)
             container += output_area
         else:
             text = '\n'.join(self.content.data)
-            node = CodeNode.create(text)
+            node = CodeNode.create(text, classes=['output_area'])
             _set_empty_lines(node, self.options)
             node.attributes['latex_prompt'] = latex_prompt
             container += node
@@ -1038,6 +1090,7 @@ def builder_inited(app):
     if app.config.nbsphinx_prompt_width is None:
         app.config.nbsphinx_prompt_width = {
             'agogo': '7ex',
+            'alabaster': '8ex',
             'better': '8ex',
             'classic': '7ex',
             'cloud': '8ex',
@@ -1101,11 +1154,13 @@ def depart_code_html(self, node):
 def visit_code_latex(self, node):
     """Avoid creating a separate prompt node.
 
-    The prompt will be pre-pended in the main code node.
+    The prompt (which is stored in the "latex_prompt" attribute) will be
+    pre-pended in the main code node.
 
     """
     if 'latex_prompt' not in node.attributes:
         raise docutils.nodes.SkipNode()
+    self.pushbody([])  # See popbody() below
 
 
 def depart_code_latex(self, node):
@@ -1116,7 +1171,7 @@ def depart_code_latex(self, node):
     * Add prompt to the first line, empty space to the following lines
 
     """
-    lines = self.body[-1].split('\n')
+    lines = ''.join(self.popbody()).split('\n')
     out = []
     assert lines[0] == ''
     out.append(lines[0])
@@ -1147,7 +1202,7 @@ def depart_code_latex(self, node):
         assert False
     assert lines[-1] == ''
     out.append(lines[-1])
-    self.body[-1] = '\n'.join(out)
+    self.body.append('\n'.join(out))
 
 
 def visit_admonition_html(self, node):
@@ -1211,6 +1266,7 @@ def setup(app):
     app.add_config_value('nbsphinx_codecell_lexer', 'none', rebuild='env')
     # Default value is set in builder_inited():
     app.add_config_value('nbsphinx_prompt_width', None, rebuild='html')
+    app.add_config_value('nbsphinx_responsive_width', '540px', rebuild='html')
 
     app.add_directive('nbinput', NbInput)
     app.add_directive('nboutput', NbOutput)
